#!/bin/bash

CHECK_DEVICES=`lsblk -l -o NAME,TYPE,PKNAME | grep part | grep -v "^\`mount | grep ^\/dev | awk -F ' ' '{print $1}' | awk -F '/' '{print $3}'\`" | awk -F ' ' '{print "/dev/"$3}'`

if [ "$CHECK_DEVICES" != "" ];
then

BASE_NAME=`basename -s .sh $0`
BASE_PATH=`cd \`dirname $0\` 1>/dev/null 2>/dev/null && pwd`

if [ "$BASE_NAME" != "" ] && [ "$BASE_PATH" != "" ];
then

CONF_PATH=`[ -f "${BASE_PATH}/${BASE_NAME}.conf" ] && echo "${BASE_PATH}/${BASE_NAME}.conf"`

if [ "$CONF_PATH" != "" ];
then

DEFAULT_REMOTE_SSH_HOST=`cat "$CONF_PATH" 2>/dev/null | grep DEFAULT_REMOTE_SSH_HOST | awk -F '=' '{print $2}' | awk -F ' ' '{print $1}'`
DEFAULT_REMOTE_SSH_PORT=`cat "$CONF_PATH" 2>/dev/null | grep DEFAULT_REMOTE_SSH_PORT | awk -F '=' '{print $2}' | awk -F ' ' '{print $1}'`
DEFAULT_REMOTE_SSH_USER=`cat "$CONF_PATH" 2>/dev/null | grep DEFAULT_REMOTE_SSH_USER | awk -F '=' '{print $2}' | awk -F ' ' '{print $1}'`
DEFAULT_REMOTE_SSH_PATH=`cat "$CONF_PATH" 2>/dev/null | grep DEFAULT_REMOTE_SSH_PATH | awk -F '=' '{print $2}' | awk -F ' ' '{print $1}'`

DEFAULT_SYSTEM_ID=`cat /proc/cpuinfo 2>/dev/null | grep Serial | awk -F ' ' '{print $3}'`

if [ "$DEFAULT_SYSTEM_ID" == "" ];
then
DEFAULT_SYSTEM_ID=`hostname`
fi

if [ "$DEFAULT_REMOTE_SSH_HOST" != "" ] && [ "$DEFAULT_REMOTE_SSH_PORT" != "" ] && [ "`(echo $DEFAULT_REMOTE_SSH_PORT | grep "[^0-9]" 1>/dev/null) || echo OK`" == "OK" ] && [ "$DEFAULT_REMOTE_SSH_USER" != "" ] && [ "$DEFAULT_REMOTE_SSH_PATH" != "" ] && [ "$DEFAULT_SYSTEM_ID" != "" ];
then

PS3="Select device: "

while true; do

clear

echo "List of block devices:"
lsblk

echo ""
echo "List of available devices:"

DEVICES=(`lsblk -l -o NAME,TYPE,PKNAME | grep part | grep -v "^\`mount | grep ^\/dev | awk -F ' ' '{print $1}' | awk -F '/' '{print $3}'\`" | awk -F ' ' '{print "/dev/"$3}'`)

select DEVICE in "${DEVICES[@]}"; do 
break
done

if [ "$DEVICE" != "" ] && [ -b $DEVICE ];
then
echo Selected device: $DEVICE
lsblk $DEVICE

while [ "$YN_CONTINUE" == "" ] || ( [ "$YN_CONTINUE" != "Y" ] && [ "$YN_CONTINUE" != "y" ] && [ "$YN_CONTINUE" != "N" ] && [ "$YN_CONTINUE" != "n" ] ); do
read -e -p "Are you sure to continue? [y/N]: " YN_CONTINUE
if [ "$YN_CONTINUE" == "" ];
then
YN_CONTINUE=N
fi
done
if [ "$YN_CONTINUE" != "Y" ] && [ "$YN_CONTINUE" != "y" ];
then
break
fi
unset YN_CONTINUE

while [ "$REMOTE_SSH_HOST" == "" ]; do
read -e -p "Enter SSH hostname [$DEFAULT_REMOTE_SSH_HOST]: " REMOTE_SSH_HOST
if [ "$REMOTE_SSH_HOST" == "" ];
then
REMOTE_SSH_HOST=$DEFAULT_REMOTE_SSH_HOST
fi
done

while [ "$REMOTE_SSH_PORT" == "" ] || [ "`(echo $REMOTE_SSH_PORT | grep "[^0-9]" 1>/dev/null) || echo OK`" != "OK" ]; do
read -e -p "Enter SSH port [$DEFAULT_REMOTE_SSH_PORT]: " REMOTE_SSH_PORT
if [ "$REMOTE_SSH_PORT" == "" ];
then
REMOTE_SSH_PORT=$DEFAULT_REMOTE_SSH_PORT
fi
done

while [ "$REMOTE_SSH_USER" == "" ]; do
read -e -p "Enter SSH user [$DEFAULT_REMOTE_SSH_USER]: " REMOTE_SSH_USER
if [ "$REMOTE_SSH_USER" == "" ];
then
REMOTE_SSH_USER=$DEFAULT_REMOTE_SSH_USER
fi
done

while [ "$REMOTE_SSH_PATH" == "" ]; do
read -e -p "Enter SSH remote path [$DEFAULT_REMOTE_SSH_PATH]: " REMOTE_SSH_PATH
if [ "$REMOTE_SSH_PATH" == "" ];
then
REMOTE_SSH_PATH=$DEFAULT_REMOTE_SSH_PATH
fi
done

while [ "$SYSTEM_ID" == "" ]; do
read -e -p "Enter system id [$DEFAULT_SYSTEM_ID]: " SYSTEM_ID
if [ "$SYSTEM_ID" == "" ];
then
SYSTEM_ID=$DEFAULT_SYSTEM_ID
fi
done

IMAGE_ID=`date +%F_%s`
DEV_SIZE=`cat /proc/partitions 2>/dev/null | grep "\`echo "${DEVICE}" | awk -F '/' '{print $3}'\`$" | awk -F ' ' '{print $3}'`

if [ "$IMAGE_ID" == "" ] || [ "$DEV_SIZE" == "" ] || [ "`(echo $DEV_SIZE | grep "[^0-9]" 1>/dev/null) || echo OK`" != "OK" ];
then
break
fi

VALID_REMOTE_SSH_PATH=`ssh -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -o BatchMode=yes -o PasswordAuthentication=no -o LogLevel=ERROR -p ${REMOTE_SSH_PORT} ${REMOTE_SSH_USER}@${REMOTE_SSH_HOST} "cd ${REMOTE_SSH_PATH} 1>/dev/null 2>/dev/null && pwd"`

if [ "$VALID_REMOTE_SSH_PATH" != "" ];
then

while [ "$YN_CONTINUE" == "" ] || ( [ "$YN_CONTINUE" != "Y" ] && [ "$YN_CONTINUE" != "y" ] && [ "$YN_CONTINUE" != "N" ] && [ "$YN_CONTINUE" != "n" ] ); do
read -e -p "Image of $DEVICE block device will be transferred to [ssh://$REMOTE_SSH_USER@$REMOTE_SSH_HOST:$REMOTE_SSH_PORT$VALID_REMOTE_SSH_PATH/${SYSTEM_ID}_${IMAGE_ID}.img.gz]. Are you sure to continue? [y/N]: " YN_CONTINUE
if [ "$YN_CONTINUE" == "" ];
then
YN_CONTINUE=N
fi
done
if [ "$YN_CONTINUE" != "Y" ] && [ "$YN_CONTINUE" != "y" ];
then
break
fi
unset YN_CONTINUE

echo "Reading, sending and writing image [DEVICE = $DEVICE, SYSTEM_ID = $SYSTEM_ID, IMAGE_ID = $IMAGE_ID]..."

dd if=$DEVICE | pv -s ${DEV_SIZE}K | gzip -c | ssh -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -o BatchMode=yes -o PasswordAuthentication=no -o LogLevel=ERROR -p ${REMOTE_SSH_PORT} ${REMOTE_SSH_USER}@${REMOTE_SSH_HOST} "cat - >${VALID_REMOTE_SSH_PATH}/${SYSTEM_ID}_${IMAGE_ID}.img.gz"

exit $?

else

echo "ERROR: Cannot access SSH remote path [$REMOTE_SSH_PATH]!"

fi

break

else

echo "ERROR: Invalid selection!"

fi

done

else

if [ "$DEFAULT_SYSTEM_ID" == "" ];
then

echo "ERROR: Invalid system id [$DEFAULT_SYSTEM_ID]!"

else

echo "Invalid values in configuration file [${BASE_PATH}/${BASE_NAME}.conf]!"

echo DEFAULT_REMOTE_SSH_HOST = $DEFAULT_REMOTE_SSH_HOST
echo DEFAULT_REMOTE_SSH_PORT = $DEFAULT_REMOTE_SSH_PORT
echo DEFAULT_REMOTE_SSH_USER = $DEFAULT_REMOTE_SSH_USER
echo DEFAULT_REMOTE_SSH_PATH = $DEFAULT_REMOTE_SSH_PATH

fi

fi

else

echo Missing configuration file [${BASE_PATH}/${BASE_NAME}.conf]!

fi

fi

else

echo "List of block devices:"
lsblk
echo ""
echo "No devices available!"
echo ""

fi

exit -1
